/*
 * Copyright (c) 2007 Markus Fisch <mf@markusfisch.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Markus Fisch nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _PieDock_Application_
#define _PieDock_Application_

#include "Settings.h"

#include <X11/Xlib.h>
#include <string>

namespace PieDock
{
	/**
	 * PieDock application
	 *
	 * @author Markus Fisch <mf@markusfisch.de>
	 */
	class Application
	{
		public:
			Application( int argc, char **argv );
			virtual ~Application();
			inline Display *getDisplay() const { return display; }
			inline Settings &getSettings() { return settings; }
			bool wake() const;
			int run( bool * );
			static std::string getHome();

		private:
			enum PulseBeats
			{
				StandBy = 0,
				Active = 10000
			};

			enum
			{
				WakeUp = 1,
				UnixPathMax = 108
			};

			Display *display;
			Window root;
			Settings settings;
			int suspend;
			std::string socketFile;

			void grabTriggers();
			void ungrabTriggers();
	};
}

#endif
