/*
 * Copyright (c) 2007 Markus Fisch <mf@markusfisch.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Markus Fisch nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _PieDock_Blender_
#define _PieDock_Blender_

#include "Surface.h"
#include "RgbaSurface.h"

namespace PieDock
{
	/**
	 * Blend RGBA surfaces into some other surface of arbitrary color depth
	 *
	 * @author Markus Fisch <mf@markusfisch.de>
	 */
	class Blender
	{
		public:
			enum Opacity
			{
				Transparent = 0,
				Translucent = 128,
				Opaque = 0xff
			};

			Blender( Surface & );
			virtual ~Blender() {}
			virtual void blend( const RgbaSurface &, int, int, int = Opaque );

		protected:
			Surface *canvas;

			typedef struct Details
			{
				unsigned char *dest;
				unsigned char *src;
				int length;
				int repeats;
				int srcSkip;
				int destSkip;
				int alpha;
			};

			virtual void blendInto32Bit( Details & );
			virtual void blendInto24Bit( Details & );
			virtual void blendInto16Bit( Details & );
	};
}

#endif
