/*
 * Copyright (c) 2007 Markus Fisch <mf@markusfisch.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Markus Fisch nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _PieDock_IconMap_
#define _PieDock_IconMap_

#include "Icon.h"

#include <string>
#include <map>

namespace PieDock
{
	/**
	 * Icon map
	 *
	 * @author Markus Fisch <mf@markusfisch.de>
	 */
	class IconMap
	{
		public:
			IconMap() {}
			virtual ~IconMap();
			virtual void reset();
			virtual void setPath( std::string p ) { path = p; }
			virtual const std::string &getPath() const { return path; }
			virtual void addNameAlias( std::string, std::string );
			virtual void addClassAlias( std::string, std::string );
			virtual void addTitleAlias( std::string, std::string );
			virtual Icon *getIconByName( std::string );
			virtual Icon *getIconByClass( std::string );
			virtual Icon *getIconByTitle( std::string );
			virtual Icon *getFallbackIcon( std::string );

		protected:
			typedef std::map<std::string, std::string> AliasToFile;
			typedef std::map<std::string, Icon *> FileToIcon;

			std::string path;
			AliasToFile nameToFile;
			AliasToFile classToFile;
			AliasToFile titleToFile;
			FileToIcon cache;
			static const char fallbackPng[];

			virtual Icon *createIcon( RgbaSurface &, std::string );
			virtual void freeIcons();
	};
}

#endif
