/*
 * Copyright (c) 2007 Markus Fisch <mf@markusfisch.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Markus Fisch nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _PieDock_Menu_
#define _PieDock_Menu_

#include "Application.h"
#include "Icon.h"

#include <string>
#include <vector>
#include <map>

namespace PieDock
{
	/**
	 * Abstract menu
	 *
	 * @author Markus Fisch <mf@markusfisch.de>
	 */
	class Menu
	{
		public:
			Menu( Application * );
			virtual ~Menu();
			virtual void update();
			virtual void draw( int, int ) {}
			virtual void execute( Settings::Action = Settings::Launch );

		protected:
			/**
			 * Menu item
			 */
			class MenuItem
			{
				public:
					typedef struct
					{
						Window window;
						int left;
						int top;
						int right;
						int bottom;
						int workspace;
						int state;
					} WindowAttribute;

					typedef std::vector<WindowAttribute> WindowAttributes;

					MenuItem( Icon *, bool = false );
					virtual ~MenuItem() {}
					inline Icon *getIcon() const { return icon; }
					inline WindowAttributes &getWindowAttributes() {
						return windowAttributes; }
					inline const bool hasWindows() const {
						return windowAttributes.empty()^true; }
					inline const bool isSticky() const { return sticky; }
					inline void clearWindows() { windowAttributes.clear(); }
					void addWindow( Display *, Window );
					const Window getNextWindow();
					const Window getPreviousWindow();
					const bool isUnmapped();

				private:
					WindowAttributes windowAttributes;
					Window last;
					Icon *icon;
					bool sticky;
			};

			typedef std::map<Icon *, MenuItem *> MenuItemsMap;
			typedef std::vector<MenuItem *> MenuItemsVector;

			Application *app;
			MenuItemsMap menuItemsMap;
			MenuItemsVector menuItemsVector;
			MenuItem *selected;

			virtual int run( std::string ) const;
	};
}

#endif
