/*
 * Copyright (c) 2007 Markus Fisch <mf@markusfisch.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Markus Fisch nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef _PieDock_PieMenu_
#define _PieDock_PieMenu_

#include "Application.h"
#include "Surface.h"
#include "Blender.h"
#include "Icon.h"
#include "Menu.h"

#include <vector>
#include <math.h>

namespace PieDock
{
	/**
	 * Realize a pie menu
	 *
	 * @author Markus Fisch <mf@markusfisch.de>
	 */
	class PieMenu : public Menu
	{
		public:
			PieMenu( Application *, Surface & );
			virtual ~PieMenu() {}
			virtual void update();
			virtual bool isObsolete( int, int );
			virtual void draw( int, int );
			virtual void turn( double );

		protected:
			static const double radiansPerCircle;

			Blender blender;
			int size;
			int maxRadius;
			int radius;
			double twist;
			int centerX;
			int centerY;
			int lastX;
			int lastY;

			/**
			 * Return the difference of two angles in radians; implemented
			 * here to ensure the method will be compiled inline
			 *
			 * @param a - angle in radians
			 * @param b - angle in radians
			 */
			inline virtual double getAngleDifference( double a, double b )
			{
				double c = a-b;
				double d;

				if( a > b )
					d = a-(b+radiansPerCircle);
				else
					d = a-(b-radiansPerCircle);

				if( fabs( c ) < fabs( d ) )
					return c;

				return d;
			}

			/**
			 * Recalculate angle to be within a valid range; implemented 
			 * here to ensure the method will be compiled inline
			 *
			 * @param a - angle in radians
			 */
			inline virtual double getValidAngle( double a )
			{
				if( a < -M_PI )
					a += radiansPerCircle;
				else if( a > M_PI )
					a -= radiansPerCircle;

				return a;
			}
	};
}

#endif
