/*
 * Copyright (c) 2007 Markus Fisch <mf@markusfisch.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Markus Fisch nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#include "RgbaSurfaceSizeMap.h"
#include "Resampler.h"

using namespace PieDock;

/**
 * Initialize object
 *
 * @param s - some RGBA surface
 */
RgbaSurfaceSizeMap::RgbaSurfaceSizeMap( RgbaSurface &s ) :
	surface( s )
{
}

/**
 * Clean up
 */
RgbaSurfaceSizeMap::~RgbaSurfaceSizeMap()
{
	for( SurfaceMap::iterator i = surfaceMap.begin();
		i != surfaceMap.end();
		i++ )
		delete (*i).second;

	surfaceMap.clear();
}

/**
 * Return a sized version of the icon image
 *
 * @param width - width of surface in pixels
 * @param height - height of surface in pixels
 */
const RgbaSurface *RgbaSurfaceSizeMap::getSurface( int width, int height )
{
	if( width == surface.getWidth() &&
		height == surface.getHeight() )
		return &surface;

	int format = (width<<16)+height;
	SurfaceMap::iterator i;

	if( (i = surfaceMap.find( format )) == surfaceMap.end() )
	{
		RgbaSurface *s = new RgbaSurface( width, height );

		Resampler::resample( *s, surface );
		surfaceMap.insert( std::make_pair( format, s ) );

		return s;
	}

	return (*i).second;
}
