/*
 * Copyright (c) 2007 Markus Fisch <mf@markusfisch.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Markus Fisch nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#include "TransparentWindow.h"

using namespace PieDock;

/**
 * Initialize object
 *
 * @param a - Application object
 */
TransparentWindow::TransparentWindow( Application &a ) :
	app( &a ),
	width( app->getSettings().getWidth() ),
	height( app->getSettings().getHeight() ),
	canvas( width, height, app->getDisplay() ),
	buffer( new unsigned char[canvas.getSize()] ),
	gc( XDefaultGC( app->getDisplay(), DefaultScreen( app->getDisplay() ) ) )
{
	unsigned long vmask;
    XSetWindowAttributes xswat;

    xswat.override_redirect = True;
    xswat.do_not_propagate_mask =
		KeyPressMask | KeyReleaseMask |
		ButtonPressMask | ButtonReleaseMask;
    vmask = CWOverrideRedirect | CWDontPropagate;

	window = XCreateWindow(
		app->getDisplay(),
		DefaultRootWindow( app->getDisplay() ),
		0,
		0,
		width,
		height,
		0,
		CopyFromParent,
		CopyFromParent,
		CopyFromParent,
		vmask, &xswat );

	// you still need to select input
}

/**
 * Clean up
 */
TransparentWindow::~TransparentWindow()
{
	// it's valid to delete 0
	delete buffer;

	XDestroyWindow( app->getDisplay(), window );
}

/**
 * Show window; the caller must ensure that the window is completely 
 * visible on the screen or XGetSubImage will fail !
 */
void TransparentWindow::show()
{
	XMapRaised( app->getDisplay(), window );

	XGetSubImage(
		app->getDisplay(),
		window,
		0,
		0,
		width,
		height,
		0xffffffff,
		ZPixmap,
		canvas.getResource(),
		0,
		0 );

	memcpy( buffer,
		canvas.getData(),
		canvas.getSize() );
}

/**
 * Hide window
 */
void TransparentWindow::hide() const
{
	XUnmapWindow( app->getDisplay(), window );
}

/**
 * Clear window for drawing
 */
void TransparentWindow::clear()
{
	memcpy(
		canvas.getData(),
		buffer,
		canvas.getSize() );
}

/**
 * Update window
 */
void TransparentWindow::update() const
{
	XPutImage(
		app->getDisplay(),
		window,
		gc,
		canvas.getResource(),
		0,
		0,
		0,
		0,
		width,
		height );
}
