/*
 * Copyright (c) 2007 Markus Fisch <mf@markusfisch.de>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of Markus Fisch nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */
#include "WildcardCompare.h"

#include <string>

using namespace PieDock;

/**
 * Returns if pattern matches string
 *
 * @param literal - string to match
 * @param pattern - pattern with optional wildcard characters (*?)
 * @param caseSensitive - true if matching should be case-sensitive (optional)
 */
const bool WildcardCompare::match( const char *literal, 
	const char *pattern, bool caseSensitive )
{
	for( ; *pattern; )
	{
		switch( *pattern )
		{
			case '*':
				{
					const char *match = pattern;

					while( *match == '*' || *match == '?' )
						match++;

					if( !*match )
						return true;

					const char *wildcard;
					char save;
					size_t length;

					if( (length = strcspn( match, "*?" )) )
					{
						wildcard = match+length;
						save = *wildcard;
						*((char *) wildcard) = 0;
					}

					const char *last = NULL;
					const char *pos;

					if( caseSensitive )
						for( pos = literal;
							(pos = strstr( pos, match ));
							pos++ )
							last = pos;
					else
						for( pos = literal;
							(pos = strcasestr( pos, match ));
							pos++ )
							last = pos;

					if( wildcard )
						*((char *) wildcard) = save;

					if( !last )
						return false;

					literal = last+length;

					if( !wildcard )
					{
						if( *literal )
							return false;

						return true;
					}

					pattern = wildcard;
				}
				break;
			case '?':
				if( !*(literal++) )
					return false;
				pattern++;
				break;
			default:
				if( !*literal )
					return false;

				if( caseSensitive )
				{
					if( *pattern != *literal )
						return false;
				}
				else
				{
					register unsigned char p = *pattern;
					register unsigned char l = *literal;

					if( p > 64 && p < 91 )
						p += 32;
					if( l > 64 && l < 91 )
						l += 32;

					if( l != p )
						return false;
				}

				literal++;
				pattern++;
				break;
		}
	}

	if( *literal )
		return false;

	return true;
}
